@@ -1,11 +1,14 @@
 package Test::Compile;
+
 use 5.006;
 use warnings;
 use strict;
+
 use Test::Builder;
 use File::Spec;
 use UNIVERSAL::require;
-our $VERSION = '0.11';
+
+our $VERSION = '0.17';
 my $Test = Test::Builder->new;
 
 sub import {
@@ -27,27 +30,12 @@ sub import {
 sub pm_file_ok {
     my $file = shift;
     my $name = @_ ? shift : "Compile test for $file";
-    if (!-f $file) {
-        $Test->ok(0, $name);
-        $Test->diag("$file does not exist");
-        return;
-    }
-    my $module = $file;
-    $module =~ s!^(blib[/\\])?lib[/\\]!!;
-    $module =~ s![/\\]!::!g;
 
-    $module =~ s/\.pm$//;
-    my $ok = 1;
-    $module->use;
-    $ok = 0 if $@;
-    my $diag = '';
+    my $ok = _run_in_subprocess(sub{_check_syntax($file,1)});
 
-    unless ($ok) {
-        $diag = "couldn't use $module ($file): $@";
-    }
     $Test->ok($ok, $name);
-    $Test->diag($diag) unless $ok;
-    $ok;
+    $Test->diag("$file does not compile") unless $ok;
+    return $ok;
 }
 
 sub pl_file_ok {
@@ -67,20 +55,12 @@ sub pl_file_ok {
             return;
         }
     }
-    unless (-f $file) {
-        $Test->ok(0, $name);
-        $Test->diag("$file does not exist");
-        return;
-    }
-    my $out = `$^X -cw $file 2>&1`;
-    if ($?) {
-        $Test->ok(0, 'Script does not compile');
-        $Test->diag($out);
-        return;
-    } else {
-        $Test->ok(1, $name);
-        return 1;
-    }
+
+    my $ok = _run_in_subprocess(sub{_check_syntax($file,0)});
+
+    $Test->ok($ok, $name);
+    $Test->diag("$file does not compile") unless $ok;
+    return $ok;
 }
 
 sub all_pm_files_ok {
@@ -95,6 +75,7 @@ sub all_pm_files_ok {
 
 sub all_pl_files_ok {
     my @files = @_ ? @_ : all_pl_files();
+    $Test->skip_all("no pl files found") unless @files;
     $Test->plan(tests => scalar @files);
     my $ok = 1;
     for (@files) {
@@ -105,38 +86,72 @@ sub all_pl_files_ok {
 
 sub all_pm_files {
     my @queue = @_ ? @_ : _pm_starting_points();
+
     my @pm;
-    while (@queue) {
-        my $file = shift @queue;
-        if (-d $file) {
-            local *DH;
-            opendir DH, $file or next;
-            my @newfiles = readdir DH;
-            closedir DH;
-            @newfiles = File::Spec->no_upwards(@newfiles);
-            @newfiles = grep { $_ ne "CVS" && $_ ne ".svn" } @newfiles;
-            for my $newfile (@newfiles) {
-                my $filename = File::Spec->catfile($file, $newfile);
-                if (-f $filename) {
-                    push @queue, $filename;
-                } else {
-                    push @queue, File::Spec->catdir($file, $newfile);
-                }
-            }
-        }
+    for my $file ( _find_files(@queue) ) {
         if (-f $file) {
             push @pm, $file if $file =~ /\.pm$/;
         }
     }
-    @pm;
+    return @pm;
 }
 
 sub all_pl_files {
     my @queue = @_ ? @_ : _pl_starting_points();
+
     my @pl;
-    while (@queue) {
-        my $file = shift @queue;
-        if (-d $file) {
+    for my $file ( _find_files(@queue) ) {
+        if (defined($file) && -f $file) {
+            # Only accept files with no extension or extension .pl
+            push @pl, $file if $file =~ /(?:^[^.]+$|\.pl$)/;
+        }
+    }
+    return @pl;
+}
+
+sub _run_in_subprocess {
+    my ($closure) = @_;
+
+    my $pid = fork();
+    if ( ! defined($pid) ) {
+        return 0;
+    } elsif ( $pid ) {
+        wait();
+        return ($? ? 0 : 1);
+    } else {
+        close(STDERR);
+        my $rv = $closure->();
+        exit ($rv ? 0 : 1);
+    }
+}
+
+sub _check_syntax {
+    my ($file,$require) = @_;
+
+    if (-f $file) {
+        if ( $require ) {
+            my $module = $file;
+            $module =~ s!^(blib[/\\])?lib[/\\]!!;
+            $module =~ s![/\\]!::!g;
+            $module =~ s/\.pm$//;
+    
+            $module->use;
+            return ($@ ? 0 : 1);
+        } else {
+            my @perl5lib = split(':', ($ENV{PERL5LIB}||""));
+            my $taint = _is_in_taint_mode($file);
+            unshift @perl5lib, 'blib/lib';
+            system($^X, (map { "-I$_" } @perl5lib), "-c$taint", $file);
+            return ($? ? 0 : 1);
+        }
+    }
+}
+
+sub _find_files {
+    my (@queue) = @_;
+
+    for my $file (@queue) {
+        if (defined($file) && -d $file) {
             local *DH;
             opendir DH, $file or next;
             my @newfiles = readdir DH;
@@ -152,13 +167,8 @@ sub all_pl_files {
                 }
             }
         }
-        if (-f $file) {
-
-            # Only accept files with no extension or extension .pl
-            push @pl, $file if $file =~ /(?:^[^.]+$|\.pl$)/;
-        }
     }
-    @pl;
+    return @queue;
 }
 
 sub _pm_starting_points {
@@ -170,11 +180,22 @@ sub _pl_starting_points {
     return 'script' if -e 'script';
     return 'bin'    if -e 'bin';
 }
+
+sub _is_in_taint_mode {
+    my $file = shift;
+    open(FILE, $file) or die "could not open $file";
+    my $shebang = <FILE>;
+    my $taint = "";
+    if ($shebang =~ /^#![\/\w]+\s+\-w?([tT])/) {
+        $taint = $1;
+    }
+    close FILE;
+    return $taint;
+}
+
 1;
 __END__
 
-=for stopwords Sagar Shah
-
 =head1 NAME
 
 Test::Compile - Check whether Perl module files compile correctly
@@ -334,34 +355,22 @@ sorted, you'll have to sort them yourself.
 
 =back
 
-=head1 BUGS AND LIMITATIONS
-
-No bugs have been reported.
-
-Please report any bugs or feature requests through the web interface at
-L<http://rt.cpan.org>.
-
-=head1 INSTALLATION
-
-See perlmodinstall for information and options on installing Perl modules.
-
-=head1 AVAILABILITY
-
-The latest version of this module is available from the Comprehensive Perl
-Archive Network (CPAN). Visit <http://www.perl.com/CPAN/> to find a CPAN
-site near you. Or see L<http://search.cpan.org/dist/Test-Compile/>.
-
 =head1 AUTHORS
 
-Sagar R. Shah C<< <srshah@cpan.org> >>
-
-Marcel GrE<uuml>nauer, C<< <marcel@cpan.org> >>
+Sagar R. Shah C<< <srshah@cpan.org> >>,
+Marcel GrE<uuml>nauer, C<< <marcel@cpan.org> >>,
+Evan Giles, C<< <egiles@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2007-2009 by the authors.
+Copyright 2007-2012 by the authors.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
+=head1 SEE ALSO
+
+L<Test::LoadAllModules> just handles modules, not script files, but has more
+fine-grained control.
+
 =cut
